import { world, system } from '@minecraft/server';

// Track players who are currently using their items
const playersUsingItems = new Set();

// Track accumulation progress for players (0-26)
const playerAccumulationProgress = new Map();

// Track action bar display intervals for cleanup
const actionBarIntervals = new Map();

// Track players who should have action bar displayed
const playersWithActionBar = new Map();

// Track players who are in pull cooldown (resetting accumulation)
const playersPullCooldown = new Set();

// Track players currently in pull countdown
const playersInPullCountdown = new Set();

// Track active pull effects
const activePullEffects = new Map();

// Track when players started accumulating (for natural progression)
const playerAccumulationStartTime = new Map();

// Constants for accumulation
const ACCUMULATION_DURATION_TICKS = 3600; // 3 minutes = 180 seconds = 3600 ticks
const MAX_ACCUMULATION = 26; // Match your original bar length

// Function to add tag based on accumulation level
function addAccumulationTag(player, level) {
    const tagName = `amarite_longsword_hits_${level}`;

    // Remove previous accumulation tags
    for (let i = 0; i < level; i++) {
        const previousTag = `amarite_longsword_hits_${i}`;
        if (player.hasTag(previousTag)) {
            player.removeTag(previousTag);
        }
    }

    // Add the new accumulation tag
    player.addTag(tagName);
    console.log(`Added tag "${tagName}" to player ${player.name}`);
}

// Function to clear all accumulation tags for a player
function clearAllAccumulationTags(player) {
    // Clear tags from 0 to maximum
    for (let i = 0; i <= MAX_ACCUMULATION; i++) {
        const tagName = `amarite_longsword_hits_${i}`;
        if (player.hasTag(tagName)) {
            player.removeTag(tagName);
        }
    }
}

// Function to clear action bar for a player
function clearActionBar(player) {
    if (actionBarIntervals.has(player.id)) {
        system.clearRun(actionBarIntervals.get(player.id));
        actionBarIntervals.delete(player.id);
    }
    playersWithActionBar.delete(player.id);
    player.onScreenDisplay.setActionBar('');
}

const playersHeardChargedSound = new Set();

// Function to get action bar text based on accumulation level with color progression - YOUR ORIGINAL DESIGN
function getActionBarText(accumulationLevel, player = null) {
    // Define color codes
    const GRAY = "§7";
    const PURPLE = "§5";
    const YELLOW = "§e";
    const RESET = "§r";

    // Define accumulation thresholds for color changes
    const PURPLE_THRESHOLD = 8;  // Purple starts at 8 accumulation
    const YELLOW_THRESHOLD = 16; // Yellow starts at 16 accumulation

    // Determine color based on accumulation level
    let color = GRAY;
    if (accumulationLevel >= YELLOW_THRESHOLD) {
        color = YELLOW;
    } else if (accumulationLevel >= PURPLE_THRESHOLD) {
        color = PURPLE;
    }

    // Create visual bar representation - YOUR ORIGINAL DESIGN
    const maxBarLength = 26; // Maximum visual bar length
    const filledLength = Math.min(Math.floor(accumulationLevel), maxBarLength);
    const emptyLength = maxBarLength - filledLength;

    // Create the bar with appropriate colors
    let bar = color;
    for (let i = 0; i < filledLength; i++) {
        bar += "█";
    }
    bar += GRAY;
    for (let i = 0; i < emptyLength; i++) {
        bar += "░";
    }
    bar += RESET;

    // Your original messages for each accumulation level
    if (accumulationLevel === 0) {
        if (player) {
            player.runCommand(`/enchant @s mending`);
            playersHeardChargedSound.delete(player.id);
        }
        return ``;

    } else if (accumulationLevel === 1) {
        if (player) {
            player.runCommand(`/enchant @s mending`);
        }
        return ``;

    } else if (accumulationLevel === 2) {
        if (player) {
            player.runCommand(`/enchant @s mending`);
        }
        return ``;

    } else if (accumulationLevel === 3) {
        if (player) {
            player.runCommand(`/enchant @s mending`);
        }
        return ``;

    } else if (accumulationLevel === 4) {
        if (player) {
            player.runCommand(`/enchant @s mending`);
        }
        return ``;

    } else if (accumulationLevel === 5) {
        if (player) {
            player.runCommand(`/enchant @s mending`);
        }
        return ``;

    } else if (accumulationLevel === 6) {
        if (player) {
            player.runCommand(`/enchant @s mending`);
        }
        return ``;

    } else if (accumulationLevel === 7) {
        if (player) {
            player.runCommand(`/enchant @s mending`);
        }
        return ``;

    } else if (accumulationLevel === 8) {
        if (player) {
            player.runCommand(`/enchant @s mending`);
        }
        return ``;

    } else if (accumulationLevel === 9) {
        if (player) {
            player.runCommand(`/enchant @s mending`);
        }
        return ``;

    } else if (accumulationLevel === 10) {
        if (player) {
            player.runCommand(`/enchant @s mending`);
        }
        return ``;

    } else if (accumulationLevel === 11) {
        if (player) {
            player.runCommand(`/enchant @s mending`);
        }
        return ``;

    } else if (accumulationLevel === 12) {
        if (player) {
            player.runCommand(`/enchant @s mending`);
        }
        return ``;

    } else if (accumulationLevel === 13) {
        if (player) {
            player.runCommand(`/enchant @s mending`);
        }
        return ``;

    } else if (accumulationLevel === 14) {
        if (player) {
            player.runCommand(`/enchant @s mending`);
        }
        return ``;

    } else if (accumulationLevel === 15) {
        if (player) {
            player.runCommand(`/enchant @s mending`);
        }
        return ``;

    } else if (accumulationLevel === 16) {
        if (player) {
            player.runCommand(`/enchant @s mending`);
        }
        return ``;

    } else if (accumulationLevel === 17) {
        if (player) {
            player.runCommand(`/enchant @s mending`);
        }
        return ``;

    } else if (accumulationLevel === 18) {
        if (player) {
            player.runCommand(`/enchant @s mending`);
        }
        return ``;

    } else if (accumulationLevel === 19) {
        if (player) {
            player.runCommand(`/enchant @s mending`);
        }
        return ``;

    } else if (accumulationLevel === 20) {
        if (player) {
            player.runCommand(`/enchant @s mending`);
        }
        return ``;

    } else if (accumulationLevel === 21) {
        if (player) {
            player.runCommand(`/enchant @s mending`);
        }
        return ``;

    } else if (accumulationLevel === 22) {
        if (player) {
            player.runCommand(`/enchant @s mending`);
        }
        return ``;

    } else if (accumulationLevel === 23) {
        if (player) {
            player.runCommand(`/enchant @s mending`);
        }
        return ``;

    } else if (accumulationLevel === 24) {
        if (player) {
            player.runCommand(`/enchant @s mending`);
        }
        return ``;

    } else if (accumulationLevel === 25) {
        if (player) {
            player.runCommand(`/enchant @s mending`);
        }
        return ``;

    } else if (accumulationLevel === 26) {
        if (player && !playersHeardChargedSound.has(player.id)) {
            player.runCommand(`/enchant @s mending`);
            player.runCommand(`/playsound Longsword_Charged @a[r=6]`);
            playersHeardChargedSound.add(player.id);
        }
        return ``;
    }

    // For accumulation levels beyond 26 - only play sound once when first reaching max
    if (player && !playersHeardChargedSound.has(player.id)) {
        player.runCommand(`/enchant @s mending`);
        player.runCommand(`/playsound Longsword_Charged @a[r=6]`);
        playersHeardChargedSound.add(player.id);
    }
    return ``;
}

// Function to get player accumulation level
function getPlayerAccumulationLevel(player) {
    return playerAccumulationProgress.get(player.id) || 0;
}

// Function to update player accumulation naturally over time
function updatePlayerAccumulation(player) {
    const currentTime = system.currentTick;
    const startTime = playerAccumulationStartTime.get(player.id);
    
    if (!startTime) {
        // Start accumulation timer
        playerAccumulationStartTime.set(player.id, currentTime);
        return 0;
    }

    // Calculate progress based on time elapsed
    const timeElapsed = currentTime - startTime;
    const progress = Math.min((timeElapsed / ACCUMULATION_DURATION_TICKS) * MAX_ACCUMULATION, MAX_ACCUMULATION);
    
    // Update accumulation progress
    playerAccumulationProgress.set(player.id, progress);
    
    // Update tags
    const accumulationLevel = Math.floor(progress);
    addAccumulationTag(player, accumulationLevel);
    
    return progress;
}

// Function to reset player accumulation
function resetPlayerAccumulation(player) {
    playerAccumulationProgress.set(player.id, 0);
    playerAccumulationStartTime.set(player.id, system.currentTick);
    clearAllAccumulationTags(player);
    addAccumulationTag(player, 0);
    playersHeardChargedSound.delete(player.id);
    console.log(`Reset accumulation for player ${player.name}`);
}

// Function to get the entity the player is looking at within range
function getTargetEntity(player, maxDistance = 60) {
    try {
        const viewDirection = player.getViewDirection();
        const playerLocation = player.location;
        const playerEyeHeight = 1.6; // Approximate eye height

        // Get all entities in the area
        const nearbyEntities = player.dimension.getEntities({
            location: playerLocation,
            maxDistance: maxDistance,
            excludeTypes: ['minecraft:item', 'minecraft:xp_orb', 'minecraft:arrow']
        });

        let closestEntity = null;
        let closestDistance = maxDistance;
        let bestDotProduct = 0.8; // Minimum dot product for "looking at" (about 36 degrees)

        for (const entity of nearbyEntities) {
            // Skip the player themselves
            if (entity.id === player.id) continue;

            const entityLocation = entity.location;
            const distance = Math.sqrt(
                Math.pow(entityLocation.x - playerLocation.x, 2) +
                Math.pow(entityLocation.y - (playerLocation.y + playerEyeHeight), 2) +
                Math.pow(entityLocation.z - playerLocation.z, 2)
            );

            // Skip if too far
            if (distance > maxDistance) continue;

            // Calculate direction from player to entity
            const directionToEntity = {
                x: (entityLocation.x - playerLocation.x) / distance,
                y: (entityLocation.y - (playerLocation.y + playerEyeHeight)) / distance,
                z: (entityLocation.z - playerLocation.z) / distance
            };

            // Calculate dot product to see if player is looking at entity
            const dotProduct =
                viewDirection.x * directionToEntity.x +
                viewDirection.y * directionToEntity.y +
                viewDirection.z * directionToEntity.z;

            // Check if this entity is more directly in the player's line of sight
            if (dotProduct > bestDotProduct && distance < closestDistance) {
                closestEntity = entity;
                closestDistance = distance;
                bestDotProduct = dotProduct;
            }
        }

        return closestEntity;
    } catch (error) {
        console.warn('Error getting target entity:', error);
        return null;
    }
}

// Function to perform pull countdown from current accumulation to zero
function performPullCountdown(player) {
    const startingAccumulation = getPlayerAccumulationLevel(player);
    let currentAccumulation = startingAccumulation;

    // Mark player as in pull countdown
    playersInPullCountdown.add(player.id);

    console.log(`Starting pull countdown for ${player.name} from ${startingAccumulation} to 0`);

    // Create countdown interval
    const countdownInterval = system.runInterval(() => {
        try {
            // Decrease accumulation
            currentAccumulation--;
            if (currentAccumulation < 0) currentAccumulation = 0;

            // Update player's accumulation
            playerAccumulationProgress.set(player.id, currentAccumulation);

            // Clear old tags and add new tag
            clearAllAccumulationTags(player);
            addAccumulationTag(player, currentAccumulation);

            // Update action bar if player is still holding the sword
            const equipment = player.getComponent('minecraft:equippable');
            if (equipment) {
                const mainHandItem = equipment.getEquipment('Mainhand');
                if (mainHandItem && mainHandItem.typeId === 'amarite:amarite_longsword=accumulate') {
                    playersWithActionBar.set(player.id, currentAccumulation);
                }
            }

            console.log(`Player ${player.name} pull countdown: ${currentAccumulation}`);

            // Check if countdown is complete
            if (currentAccumulation <= 0) {
                // Clear the interval
                system.clearRun(countdownInterval);

                // Remove from countdown tracking
                playersInPullCountdown.delete(player.id);

                // Remove from pull cooldown
                playersPullCooldown.delete(player.id);

                // Reset accumulation timer
                resetPlayerAccumulation(player);

                console.log(`Player ${player.name} pull countdown completed`);
            }
        } catch (error) {
            console.warn('Error in pull countdown:', error);
            system.clearRun(countdownInterval);
            playersInPullCountdown.delete(player.id);
            playersPullCooldown.delete(player.id);
        }
    }, 5); // Countdown every 2 ticks(Drankness is to fast)
}

// Function to perform entity pull
function performEntityPull(player) {
    const accumulationLevel = Math.floor(playerAccumulationProgress.get(player.id) || 0);

    // Check if player has enough accumulation (at least 1)
    if (accumulationLevel < 1) {
        return false;
    }

    // Check if player is already in pull cooldown or countdown
    if (playersPullCooldown.has(player.id) || playersInPullCountdown.has(player.id)) {
        return false;
    }

    // Get the entity the player is looking at
    const targetEntity = getTargetEntity(player, 60);
    if (!targetEntity) {
        console.log(`Player ${player.name} is not looking at any valid entity within range`);
        return false;
    }

    try {
        // Play sound effect
        player.runCommand(`playsound Longsword_Accumulates @a[r=6]`);

        // Calculate pull strength based on accumulation level
        const basePullStrength = Math.min(0.15 + (accumulationLevel * 0.025), 0.8);
        const verticalPullStrength = Math.min(0.1 + (accumulationLevel * 0.02), 0.6);

        // Add to pull cooldown
        playersPullCooldown.add(player.id);

        // Store pull effect data
        const pullEffectId = `${player.id}_${targetEntity.id}_${Date.now()}`;
        activePullEffects.set(pullEffectId, {
            player: player,
            target: targetEntity,
            strength: basePullStrength,
            verticalStrength: verticalPullStrength,
            damageTimer: 0,
            spiralAngle: 0,
            initialTargetId: targetEntity.id,
            targetLocked: true,
            accumulationLevel: accumulationLevel
        });

        // Start accumulation countdown FIRST
        performPullCountdown(player);

        // Create pull effect interval (keeping your original pull mechanics)
const pullInterval = system.runInterval(() => {
    try {
        const pullData = activePullEffects.get(pullEffectId);
        if (!pullData) {
            system.clearRun(pullInterval);
            return;
        }

        // Get all entities within particle range (about 8 blocks from player)
        const nearbyEntities = pullData.player.dimension.getEntities({
            location: pullData.player.location,
            maxDistance: 8,
            excludeTypes: ['minecraft:item', 'minecraft:xp_orb', 'minecraft:arrow']
        });

                // Pull all nearby entities (except the player)
        for (const entity of nearbyEntities) {
            if (entity.id === pullData.player.id) continue;
            if (!entity.isValid()) continue;

            const playerLocation = pullData.player.location;
            const entityLocation = entity.location;

            const deltaX = playerLocation.x - entityLocation.x;
            const deltaY = playerLocation.y - entityLocation.y;
            const deltaZ = playerLocation.z - entityLocation.z;
            
            const totalDistance = Math.sqrt(deltaX * deltaX + deltaY * deltaY + deltaZ * deltaZ);

            if (totalDistance >= 1.5) {
                const pullDirection = {
                    x: deltaX / totalDistance,
                    y: deltaY / totalDistance,
                    z: deltaZ / totalDistance
                };

                // Slower, gentler pull force
                const gentlePullStrength = pullData.strength * 0.3; // Much slower
                const gentleVerticalStrength = pullData.verticalStrength * 0.3;

                entity.applyKnockback(
                    pullDirection.x,
                    pullDirection.z,
                    gentlePullStrength,
                    pullDirection.y * gentleVerticalStrength
                );
            }
        }
                // Check if player is still valid
                if (!pullData.player.isValid()) {
                    activePullEffects.delete(pullEffectId);
                    system.clearRun(pullInterval);
                    return;
                }

                // Continue pulling until countdown completes
                if (!playersInPullCountdown.has(pullData.player.id)) {
                    activePullEffects.delete(pullEffectId);
                    system.clearRun(pullInterval);
                    console.log(`Pull completed - countdown finished for player ${pullData.player.name}`);
                    return;
                }

                // Check if the original target is still valid
                if (!pullData.target.isValid()) {
                    console.log(`Original target is gone, stopping pull but continuing countdown for player ${pullData.player.name}`);
                    activePullEffects.delete(pullEffectId);
                    system.clearRun(pullInterval);
                    return;
                }

                const currentTarget = pullData.target;
                const playerLocation = pullData.player.location;
                const targetLocation = currentTarget.location;

                // Calculate distance
                const deltaX = playerLocation.x - targetLocation.x;
                const deltaY = playerLocation.y - targetLocation.y;
                const deltaZ = playerLocation.z - targetLocation.z;
                
                const horizontalDistance = Math.sqrt(deltaX * deltaX + deltaZ * deltaZ);
                const totalDistance = Math.sqrt(deltaX * deltaX + deltaY * deltaY + deltaZ * deltaZ);

                // Only pull if not extremely close
                if (totalDistance >= 1.5) {
                    try {
                        // Get current velocity
                        const currentVelocity = currentTarget.getVelocity();
                        
                        // Calculate desired pull direction (normalized)
                        const pullDirection = {
                            x: deltaX / totalDistance,
                            y: deltaY / totalDistance,
                            z: deltaZ / totalDistance
                        };

                        // Apply horizontal pull using knockback
                        if (horizontalDistance > 1) {
                            currentTarget.applyKnockback(
                                pullDirection.x,
                                pullDirection.z,
                                pullData.strength,
                                0 // No vertical component in knockback
                            );
                        }

                        // Apply vertical pull using velocity manipulation
                        if (Math.abs(deltaY) > 1) {
                            const verticalForce = pullDirection.y * pullData.verticalStrength;
                            
                            // Apply upward velocity if target is below player
                            if (deltaY > 0) {
                                // Target is below player - pull up
                                const newVelocity = {
                                    x: currentVelocity.x,
                                    y: Math.min(currentVelocity.y + verticalForce, 1.5), // Cap upward velocity
                                    z: currentVelocity.z
                                };
                                currentTarget.applyKnockback(0, 0, 0, verticalForce);
                            } else {
                                // Target is above player - pull down (let gravity help)
                                const downwardForce = Math.abs(verticalForce) * 0.5;
                                currentTarget.applyKnockback(0, 0, 0, -downwardForce);
                            }
                        }

                    } catch (velocityError) {
                        // Fallback to original method if velocity manipulation fails
                        console.warn('Velocity manipulation failed, using fallback:', velocityError);
                        currentTarget.applyKnockback(
                            deltaX / totalDistance,
                            deltaZ / totalDistance,
                            pullData.strength,
                            (deltaY / totalDistance) * pullData.verticalStrength
                        );
                    }
                }

                // Damage and health transfer logic (every 10 ticks = 0.5 seconds)
                pullData.damageTimer++;
                if (pullData.damageTimer >= 10) {
                    pullData.damageTimer = 0;

                    const targetHealth = currentTarget.getComponent('minecraft:health');
                    const playerHealth = pullData.player.getComponent('minecraft:health');

                    if (targetHealth && playerHealth) {
                        const damageAmount = 1;
                        const currentTargetHealth = targetHealth.currentValue;

                        const damageCauses = ['magic', 'wither', 'void', 'drowning', 'fire', 'lava'];
                        const randomCause = damageCauses[Math.floor(Math.random() * damageCauses.length)];

                        currentTarget.applyDamage(damageAmount, {
                            cause: randomCause
                        });

                        const newTargetHealth = Math.max(0, currentTargetHealth - damageAmount);
                        const actualDamage = currentTargetHealth - newTargetHealth;

                        if (actualDamage > 0) {
                            const currentPlayerHealth = playerHealth.currentValue;
                            const maxPlayerHealth = playerHealth.effectiveMax;
                            const newPlayerHealth = Math.min(currentPlayerHealth + actualDamage, maxPlayerHealth);

                            playerHealth.setCurrentValue(newPlayerHealth);

                            pullData.player.dimension.spawnParticle('heart_gain_example', {
                                x: playerLocation.x,
                                y: playerLocation.y + 2,
                                z: playerLocation.z
                            });

                            console.log(`Health transfer: ${actualDamage} from ${currentTarget.typeId} to ${pullData.player.name}`);
                        }

                        if (newTargetHealth <= 0) {
                            console.log(`Target died, stopping pull effect for ${pullData.player.name}`);
                            activePullEffects.delete(pullEffectId);
                            system.clearRun(pullInterval);
                            return;
                        }
                    }
                }

                // Enhanced damage for high accumulation levels (using your original threshold of 16)
                if (pullData.accumulationLevel >= 16 && pullData.damageTimer % 5 === 0) {
                    const targetHealth = currentTarget.getComponent('minecraft:health');
                    if (targetHealth) {
                        currentTarget.applyDamage(0.5, {
                            cause: 'void'
                        });

                        currentTarget.dimension.spawnParticle('minecraft:dragon_breath_fire', {
                            x: currentTarget.location.x,
                            y: currentTarget.location.y + 1,
                            z: currentTarget.location.z
                        });
                    }
                }

                // Update spiral angle for particles
                pullData.spiralAngle += 0.3;

                // Your original particle effects
                const particleCount = 8;
                for (let i = 0; i < particleCount; i++) {
                    const t = (i + 1) / (particleCount + 1);
                    
                    // Create a more dynamic spiral that follows the pull direction
                    const spiralRadius = 0.3 + Math.sin(pullData.spiralAngle + i * 0.8) * 0.2;
                    
                    // Calculate the base position along the pull line
                    const baseX = targetLocation.x + (playerLocation.x - targetLocation.x) * t;
                    const baseY = targetLocation.y + (playerLocation.y - targetLocation.y) * t;
                    const baseZ = targetLocation.z + (playerLocation.z - targetLocation.z) * t;
                    
                    // Add spiral offset perpendicular to the pull direction
                    const pullDir = {
                        x: deltaX / totalDistance,
                        y: deltaY / totalDistance,
                        z: deltaZ / totalDistance
                    };
                    
                    // Create perpendicular vectors for spiral
                    const perpX = -pullDir.z;
                    const perpZ = pullDir.x;
                    const perpY = Math.sin(pullData.spiralAngle + i) * 0.3;
                    
                    const spiralOffset = {
                        x: Math.cos(pullData.spiralAngle + i * 1.2) * spiralRadius * perpX,
                        y: perpY,
                        z: Math.cos(pullData.spiralAngle + i * 1.2) * spiralRadius * perpZ
                    };

                    const particleLocation = {
                        x: baseX + spiralOffset.x,
                        y: baseY + 1 + spiralOffset.y,
                        z: baseZ + spiralOffset.z
                    };

                    pullData.player.dimension.spawnParticle('minecraft:endrod', particleLocation);
                }

                // Enhanced spiral particles around the target (more visible)
                for (let i = 0; i < 12; i++) {
                    const angle = (i / 12) * Math.PI * 2 + pullData.spiralAngle;
                    const radius = 1.0 + Math.sin(pullData.spiralAngle * 0.7 + i) * 0.4;
                    const height = Math.sin(pullData.spiralAngle * 0.5 + i * 0.5) * 0.8;

                    const particleLocation = {
                        x: targetLocation.x + Math.cos(angle) * radius,
                        y: targetLocation.y + 1.2 + height,
                        z: targetLocation.z + Math.sin(angle) * radius
                    };

                    pullData.player.dimension.spawnParticle('minecraft:endrod', particleLocation);
                }

                // Enhanced spiral around player (health absorption effect)
                for (let i = 0; i < 8; i++) {
                    const angle = (i / 8) * Math.PI * 2 - pullData.spiralAngle * 1.5;
                    const radius = 0.6 + Math.cos(pullData.spiralAngle + i) * 0.3;
                    const height = Math.cos(pullData.spiralAngle * 0.8 + i * 0.7) * 0.5;

                    const particleLocation = {
                        x: playerLocation.x + Math.cos(angle) * radius,
                        y: playerLocation.y + 1.5 + height,
                        z: playerLocation.z + Math.sin(angle) * radius
                    };

                    pullData.player.dimension.spawnParticle('particle_here', particleLocation);
                }

            } catch (error) {
                console.warn('Error in pull effect:', error);
                activePullEffects.delete(pullEffectId);
                system.clearRun(pullInterval);
            }
        }, 3); // Run every tick for smooth pulling

        console.log(`Player ${player.name} started pulling ${targetEntity.typeId} with ${accumulationLevel} accumulation level`);
        return true;

    } catch (error) {
        console.warn('Error performing entity pull:', error);
        playersPullCooldown.delete(player.id);
        return false;
    }
}

// Function to start action bar display for a player
function startActionBarDisplay(player, accumulationLevel) {
    // Clear any existing interval
    if (actionBarIntervals.has(player.id)) {
        system.clearRun(actionBarIntervals.get(player.id));
    }

    // Store that this player should have action bar displayed
    playersWithActionBar.set(player.id, accumulationLevel);

    // Set up interval to continuously display action bar while holding item
    const intervalId = system.runInterval(() => {
        try {
            // Check if player is still holding the Amarite Longsword
            const equipment = player.getComponent('minecraft:equippable');
            if (!equipment) {
                clearActionBar(player);
                return;
            }

            const mainHandItem = equipment.getEquipment('Mainhand');
            if (!mainHandItem || mainHandItem.typeId !== 'amarite:amarite_longsword=accumulate') {
                clearActionBar(player);
                return;
            }

            // Update accumulation naturally if not in countdown
            let currentAccumulation = accumulationLevel;
            if (!playersInPullCountdown.has(player.id)) {
                currentAccumulation = updatePlayerAccumulation(player);
                playersWithActionBar.set(player.id, currentAccumulation);
            } else {
                currentAccumulation = playersWithActionBar.get(player.id) || 0;
            }

            // Always show action bar when holding the sword
            player.onScreenDisplay.setActionBar(getActionBarText(Math.floor(currentAccumulation), player));
        } catch (error) {
            console.warn('Error updating action bar:', error);
            clearActionBar(player);
        }
    }, 2); // Update every 2 ticks (0.1 seconds) to prevent fading

    actionBarIntervals.set(player.id, intervalId);
}

// Subscribe to item start use events
world.afterEvents.itemStartUse.subscribe((eventData) => {
    const { source, itemStack } = eventData;

    // Check if the used item is the Amarite Longsword
    if (itemStack.typeId === 'amarite:amarite_longsword=accumulate') {
        playersUsingItems.add(source.id);
        
        // Initialize accumulation if player doesn't have any progress yet
        if (!playerAccumulationProgress.has(source.id)) {
            resetPlayerAccumulation(source);
        }
        
        // Show action bar with current accumulation level
        const currentAccumulation = playerAccumulationProgress.get(source.id) || 0;
        startActionBarDisplay(source, currentAccumulation);
        console.log(`Player ${source.name} started using Amarite Longsword`);
    }
});

// Subscribe to item stop use events
world.afterEvents.itemStopUse.subscribe((eventData) => {
    const { source, itemStack } = eventData;

    // Check if the stopped item is the Amarite Longsword
    if (itemStack && itemStack.typeId === 'amarite:amarite_longsword=accumulate') {
        playersUsingItems.delete(source.id);
        // Clear action bar display
        clearActionBar(source);
        console.log(`Player ${source.name} stopped using Amarite Longsword`);
    }
});

// Subscribe to item complete use events (backup)
world.afterEvents.itemCompleteUse.subscribe((eventData) => {
    const { source, itemStack } = eventData;

    // Check if the completed item is the Amarite Longsword
    if (itemStack.typeId === 'amarite:amarite_longsword=accumulate') {
        playersUsingItems.delete(source.id);
        // Clear action bar display
        clearActionBar(source);
        console.log(`Player ${source.name} completed using Amarite Longsword`);
    }
});

// Subscribe to entity hurt events (for damage reduction while blocking)
world.afterEvents.entityHurt.subscribe((eventData) => {
    const { hurtEntity, damageSource, damage } = eventData;

    // Check if the hurt entity is a player
    if (hurtEntity.typeId !== 'minecraft:player') {
        return;
    }

    // Check if damage source is from another entity
    if (!damageSource.damagingEntity) {
        return;
    }

    // Check if the player is currently using an item
    if (!playersUsingItems.has(hurtEntity.id)) {
        return;
    }

    // Get the player's equipment
    const equipment = hurtEntity.getComponent('minecraft:equippable');
    if (!equipment) {
        return;
    }

    // Check if player is holding the Amarite Longsword in main hand
    const mainHandItem = equipment.getEquipment('Mainhand');
    if (!mainHandItem || mainHandItem.typeId !== 'amarite:amarite_longsword=accumulate') {
        return;
    }

// Increase accumulation when blocking damage (but not during countdown)
if (!playersInPullCountdown.has(hurtEntity.id)) {
    const currentAccumulation = playerAccumulationProgress.get(hurtEntity.id) || 0;
    const newAccumulation = Math.min(currentAccumulation + 1, MAX_ACCUMULATION); // Increase by 1, cap at 26
    
    playerAccumulationProgress.set(hurtEntity.id, newAccumulation);
    
    // Update the accumulation tag
    clearAllAccumulationTags(hurtEntity);
    addAccumulationTag(hurtEntity, Math.floor(newAccumulation));
    
    // Reset the timer to account for the manual increase
    // This prevents the natural accumulation from conflicting
    const ticksForThisLevel = (newAccumulation / MAX_ACCUMULATION) * ACCUMULATION_DURATION_TICKS;
    playerAccumulationStartTime.set(hurtEntity.id, system.currentTick - ticksForThisLevel);
    
    console.log(`Player ${hurtEntity.name} accumulation increased to ${newAccumulation} from blocking damage`);
}


    console.log(`Player ${hurtEntity.name} blocked damage while using Amarite Longsword`);

    // Calculate reduced damage (half damage)
    const reducedDamage = damage * 0.5;
    const damageReduction = damage - reducedDamage;

    // Store player's current velocity to restore it (reducing knockback)
    const currentVelocity = hurtEntity.getVelocity();

    // Heal the player for the damage reduction amount
    system.run(() => {
        try {
            const healthComponent = hurtEntity.getComponent('minecraft:health');
            if (healthComponent) {
                const currentHealth = healthComponent.currentValue;
                const maxHealth = healthComponent.effectiveMax;
                const newHealth = Math.min(currentHealth + damageReduction, maxHealth);

                healthComponent.setCurrentValue(newHealth);

                // Optional: Add visual/audio feedback
                hurtEntity.runCommand(`playsound Longsword_Absorbs_Damage @a[r=6]`);

                // Update action bar display
                const currentAccumulation = playerAccumulationProgress.get(hurtEntity.id) || 0;
                startActionBarDisplay(hurtEntity, currentAccumulation);
            }

            // Reduce knockback by applying counter-velocity
            // Wait a tick to let the knockback apply first, then counter it
            system.runTimeout(() => {
                try {
                    const newVelocity = hurtEntity.getVelocity();

                    // Calculate the knockback that was applied
                    const knockbackX = newVelocity.x - currentVelocity.x;
                    const knockbackY = newVelocity.y - currentVelocity.y;
                    const knockbackZ = newVelocity.z - currentVelocity.z;

                    // Reduce knockback by 70% (keep only 30% of the knockback)
                    const knockbackReduction = 1.0;
                    const counterX = -knockbackX * knockbackReduction;
                    const counterY = -knockbackY * knockbackReduction;
                    const counterZ = -knockbackZ * knockbackReduction;

                    // Apply the counter-velocity to reduce knockback
                    hurtEntity.applyKnockback(0, 0, counterX, counterY + counterZ);

                } catch (error) {
                    console.warn('Error reducing knockback:', error);
                }
            });

        } catch (error) {
            console.warn('Error applying damage reduction:', error);
        }
    });
});

// Handle item use event for entity pull
world.afterEvents.itemUse.subscribe((eventData) => {
    const player = eventData.source;
    const item = eventData.itemStack;

    if (item?.typeId === "amarite:amarite_longsword=accumulate" && !player.hasTag("pull_cooldown")) {
        const accumulationLevel = Math.floor(getPlayerAccumulationLevel(player));

        // Check if player has at least 1 accumulation to pull AND is sneaking
        if (accumulationLevel >= 1 && player.isSneaking) {
            const pullSuccess = performEntityPull(player);

            if (pullSuccess) {
                player.addTag("pull_cooldown");

                system.runTimeout(() => {
                    player.removeTag("pull_cooldown");
                }, 10);
            }
        }
    }
});

// Subscribe to player interact events to detect left-click while right-clicking (backup method)
world.beforeEvents.itemUse.subscribe((eventData) => {
    const { source, itemStack } = eventData;

    // Check if the item is the Amarite Longsword
    if (itemStack.typeId === 'amarite:amarite_longsword=accumulate') {
        // Check if player is currently blocking (right-clicking)
        if (playersUsingItems.has(source.id)) {
            // This is a left-click while right-clicking (pull attempt)
            const accumulationLevel = Math.floor(getPlayerAccumulationLevel(source));

            // Check if player has at least 1 accumulation and no cooldown
            if (accumulationLevel >= 1 && !source.hasTag("pull_cooldown") && !playersInPullCountdown.has(source.id)) {
                const pullSuccess = performEntityPull(source);
                if (pullSuccess) {
                    // Add cooldown tag
                    source.addTag("pull_cooldown");

                    // Remove cooldown after 10 ticks
                    system.runTimeout(() => {
                        source.removeTag("pull_cooldown");
                    }, 10);

                    // Cancel the normal item use if pull was successful
                    eventData.cancel = true;
                }
            }
        }
    }
});

// Monitor players switching items to restore action bar when needed
system.runInterval(() => {
    try {
        for (const player of world.getPlayers()) {
            // Check if player is holding the sword but doesn't have active action bar
            const equipment = player.getComponent('minecraft:equippable');
            if (equipment) {
                const mainHandItem = equipment.getEquipment('Mainhand');
                if (mainHandItem && mainHandItem.typeId === 'amarite:amarite_longsword=accumulate') {
                    const hasActiveActionBar = actionBarIntervals.has(player.id);
                    if (!hasActiveActionBar) {
                        // Show action bar when holding the sword
                        const accumulationLevel = playerAccumulationProgress.get(player.id) || 0;
                        startActionBarDisplay(player, accumulationLevel);
                    }
                }
            }
        }
    } catch (error) {
        console.warn('Error in item monitoring:', error);
    }
}, 10); // Check every 10 ticks (0.5 seconds)

// Clean up intervals when players leave
world.afterEvents.playerLeave.subscribe((eventData) => {
    const { playerId } = eventData;

    // Clean up tracking data
    playersHeardChargedSound.delete(playerId);
    playersUsingItems.delete(playerId);
    playerAccumulationProgress.delete(playerId);
    playerAccumulationStartTime.delete(playerId);
    playersWithActionBar.delete(playerId);
    playersPullCooldown.delete(playerId);
    playersInPullCountdown.delete(playerId);

    // Clean up active pull effects
    for (const [effectId, pullData] of activePullEffects.entries()) {
        if (pullData.player.id === playerId) {
            activePullEffects.delete(effectId);
        }
    }

    // Clear action bar interval
    if (actionBarIntervals.has(playerId)) {
        system.clearRun(actionBarIntervals.get(playerId));
        actionBarIntervals.delete(playerId);
    }
});

// Optional: Add a function to reset accumulation and tags (useful for testing or resetting)
export function resetPlayerAccumulationLevel(player) {
    resetPlayerAccumulation(player);
    playersPullCooldown.delete(player.id);
    playersInPullCountdown.delete(player.id);
    playersHeardChargedSound.delete(player.id);

    // Clear any active pull effects for this player
    for (const [effectId, pullData] of activePullEffects.entries()) {
        if (pullData.player.id === player.id) {
            activePullEffects.delete(effectId);
        }
    }

    console.log(`Reset accumulation level for player ${player.name}`);
}

// Function to manually set accumulation level (for testing)
export function setPlayerAccumulationLevel(player, level) {
    const clampedLevel = Math.max(0, Math.min(level, MAX_ACCUMULATION));
    playerAccumulationProgress.set(player.id, clampedLevel);
    
    // Set start time to simulate natural progression to this point
    const ticksForThisLevel = (clampedLevel / MAX_ACCUMULATION) * ACCUMULATION_DURATION_TICKS;
    playerAccumulationStartTime.set(player.id, system.currentTick - ticksForThisLevel);
    
    clearAllAccumulationTags(player);
    addAccumulationTag(player, Math.floor(clampedLevel));
    
    console.log(`Set accumulation level to ${clampedLevel} for player ${player.name}`);
}
