import { world, system } from '@minecraft/server';

// Track players who are currently using their items
const playersUsingItems = new Set();

// Track hit counts for players while blocking
const playernaturalProgress = new Map();

// Track action bar display intervals for cleanup
const actionBarIntervals = new Map();

// Track players who should have action bar displayed
const playersWithActionBar = new Map();

// Track players who are in dash cooldown (resetting hit count)
const playersDashCooldown = new Set();

// Track players currently in dash countdown
const playersInDashCountdown = new Set();

// Track players who have used their first dash and are eligible for second dash
const playersEligibleForSecondDash = new Map(); // Maps player ID to remaining hit count

// Track players currently in their first dash
const playersInFirstDash = new Set();

// Track when players started accumulating (for natural progression)
const playernaturalStartTime = new Map();

const natural_DURATION_TICKS = 3600; // 3 minutes = 180 seconds = 3600 ticks
const MAX_natural = 26; // Match your original bar length

// Function to add tag based on hit count
function addnaturalTag(player, level) {
    const tagName = `amarite_longsword_hits_${level}`;

    // Remove previous natural tags
    for (let i = 0; i < level; i++) {
        const previousTag = `amarite_longsword_hits_${i}`;
        if (player.hasTag(previousTag)) {
            player.removeTag(previousTag);
        }
    }

    // Add the new natural tag
    player.addTag(tagName);
    console.log(`Added tag "${tagName}" to player ${player.name}`);
}

// Function to clear all natural tags for a player
function clearAllnaturalTags(player) {
    // Clear tags from 0 to maximum
    for (let i = 0; i <= MAX_natural; i++) {
        const tagName = `amarite_longsword_hits_${i}`;
        if (player.hasTag(tagName)) {
            player.removeTag(tagName);
        }
    }
}

// Function to clear action bar for a player
function clearActionBar(player) {
    if (actionBarIntervals.has(player.id)) {
        system.clearRun(actionBarIntervals.get(player.id));
        actionBarIntervals.delete(player.id);
    }
    playersWithActionBar.delete(player.id);
    player.onScreenDisplay.setActionBar('');
}

const playersHeardChargedSound = new Set();

// Modified getActionBarText function
function getActionBarText(naturalLevel, player = null) {
    if (naturalLevel === 0) {
        if (player) {
            player.runCommand(`/enchant @s mending`);
            // Reset sound tracking when hit count goes back to 0
            playersHeardChargedSound.delete(player.id);
        }
        return ``;
        
    } else if (naturalLevel === 1) {
        if (player) {
            player.runCommand(`/enchant @s mending`);
        }
        return ``;

    } else if (naturalLevel === 2) {
        if (player) {
            player.runCommand(`/enchant @s mending`);
        }
        return ``;

    } else if (naturalLevel === 3) {
        if (player) {
            player.runCommand(`/enchant @s mending`);
        }
        return ``;

    } else if (naturalLevel === 4) {
        if (player) {
            player.runCommand(`/enchant @s mending`);
        }
        return ``;

    } else if (naturalLevel === 5) {
        if (player) {
            player.runCommand(`/enchant @s mending`);
        }
        return ``;

    } else if (naturalLevel === 6) {
        if (player) {
            player.runCommand(`/enchant @s mending`);
        }
        return ``;
        
    } else if (naturalLevel === 7) {
        if (player) {
            player.runCommand(`/enchant @s mending`);
        }
        return ``;

    } else if (naturalLevel === 8) {
        if (player) {
            player.runCommand(`/enchant @s mending`);
        }
        return ``;

    } else if (naturalLevel === 9) {
        if (player) {
            player.runCommand(`/enchant @s mending`);
        }
        return ``;

    } else if (naturalLevel === 10) {
        if (player) {
            player.runCommand(`/enchant @s mending`);
        }
        return ``;

    } else if (naturalLevel === 11) {
        if (player) {
            player.runCommand(`/enchant @s mending`);
        }
        return ``;

    } else if (naturalLevel === 12) {
        if (player) {
            player.runCommand(`/enchant @s mending`);
        }
        return ``;

    } else if (naturalLevel === 13) {
        if (player && !playersHeardChargedSound.has(player.id)) {
            player.runCommand(`/enchant @s mending`);
            player.runCommand(`/playsound Longsword_Charged @a[r=6]`);
            playersHeardChargedSound.add(player.id);
        }
        return ``;
    }
    
    // For hit counts beyond 13 - only play sound once when first reaching 13+
    if (player && !playersHeardChargedSound.has(player.id)) {
        player.runCommand(`/enchant @s mending`);
        player.runCommand(`/playsound Longsword_Charged @a[r=6]`);
        playersHeardChargedSound.add(player.id);
    }
    return ``;
}

// Function to get player natural level
function getPlayernaturalLevel(player) {
    return Math.floor(playernaturalProgress.get(player.id) || 0);
}

// Add a new Map to track manual progress additions
const playernaturalManualBonus = new Map();

// Function to increase natural progress when blocking attacks
function increaseNaturalProgress(player, amount = 1) {
    const currentProgress = playernaturalProgress.get(player.id) || 0;
    const newProgress = Math.min(currentProgress + amount, MAX_natural);
    
    // Track the manual bonus separately
    const currentBonus = playernaturalManualBonus.get(player.id) || 0;
    playernaturalManualBonus.set(player.id, currentBonus + amount);
    
    playernaturalProgress.set(player.id, newProgress);
    
    // Update tags
    const naturalLevel = Math.floor(newProgress);
    addnaturalTag(player, naturalLevel);
    
    // Update action bar if player is holding the sword
    const equipment = player.getComponent('minecraft:equippable');
    if (equipment) {
        const mainHandItem = equipment.getEquipment('Mainhand');
        if (mainHandItem && mainHandItem.typeId === 'amarite:amarite_longsword=double_dash') {
            playersWithActionBar.set(player.id, naturalLevel);
        }
    }
    
    console.log(`Increased natural progress for ${player.name} to ${newProgress} (level ${naturalLevel})`);
    return newProgress;
}

// Function to update player natural naturally over time
function updatePlayernatural(player) {
    const currentTime = system.currentTick;
    const startTime = playernaturalStartTime.get(player.id);
    
    if (!startTime) {
        // Start natural timer
        playernaturalStartTime.set(player.id, currentTime);
        return 0;
    }

    // Calculate progress based on time elapsed
    const timeElapsed = currentTime - startTime;
    const timeBasedProgress = Math.min((timeElapsed / natural_DURATION_TICKS) * MAX_natural, MAX_natural);
    
    // Add manual bonus from blocking
    const manualBonus = playernaturalManualBonus.get(player.id) || 0;
    const totalProgress = Math.min(timeBasedProgress + manualBonus, MAX_natural);
    
    // Update natural progress
    playernaturalProgress.set(player.id, totalProgress);
    
    // Update tags
    const naturalLevel = Math.floor(totalProgress);
    addnaturalTag(player, naturalLevel);
    
    return totalProgress;
}

// Function to reset player natural
function resetPlayernatural(player) {
    playernaturalProgress.set(player.id, 0);
    playernaturalManualBonus.set(player.id, 0); // Reset manual bonus too
    playernaturalStartTime.set(player.id, system.currentTick);
    clearAllnaturalTags(player);
    addnaturalTag(player, 0);
    playersHeardChargedSound.delete(player.id);
    console.log(`Reset natural for player ${player.name}`);
}


// Function to spawn dash particles around the player
function spawnDashParticles(player, naturalLevel) {
    try {
        const playerLocation = player.location;
        const particleCount = Math.min(20 + (naturalLevel * 3), 35); // More particles based on hit count, capped at 35
        
        // Spawn particles in multiple patterns for a more dramatic effect
        for (let i = 0; i < particleCount; i++) {
            // Random offset around the player
            const offsetX = (Math.random() - 0.5) * 4; // 4 block radius
            const offsetY = Math.random() * 3; // Up to 3 blocks high
            const offsetZ = (Math.random() - 0.5) * 4; // 4 block radius
            
            const particleLocation = {
                x: playerLocation.x + offsetX,
                y: playerLocation.y + offsetY,
                z: playerLocation.z + offsetZ
            };
            
            // Spawn the minecraft:endrod particle
            player.dimension.spawnParticle('minecraft:endrod', particleLocation);
        }
        
        // Additional concentrated burst around player's center
        for (let i = 0; i < 15; i++) {
            const angle = (i / 15) * Math.PI * 2; // Circle around player
            const radius = 1.5;
            const particleLocation = {
                x: playerLocation.x + Math.cos(angle) * radius,
                y: playerLocation.y + 1 + (Math.random() * 0.5), // Around player's torso
                z: playerLocation.z + Math.sin(angle) * radius
            };
            
            player.dimension.spawnParticle('minecraft:endrod', particleLocation);
        }
        
        // Spiral effect going upward
        for (let i = 0; i < 20; i++) {
            const angle = (i / 10) * Math.PI * 4; // Double spiral
            const radius = 2 - (i * 0.15); // Decreasing radius as it goes up
            const height = i * 0.3; // Going upward
            
            const particleLocation = {
                x: playerLocation.x + Math.cos(angle) * radius,
                y: playerLocation.y + height,
                z: playerLocation.z + Math.sin(angle) * radius
            };
            
            player.dimension.spawnParticle('minecraft:endrod', particleLocation);
        }
        
        console.log(`Spawned ${particleCount + 25} minecraft:endrod particles around ${player.name} for dash`);
        
    } catch (error) {
        console.warn('Error spawning dash particles:', error);
    }
}

// Function to perform dash countdown from highest hit count to zero
function performDashCountdown(player, isSecondDash = false) {
    const startingnaturalLevel = isSecondDash ? 
        playersEligibleForSecondDash.get(player.id) || 0 : 
        getPlayernaturalLevel(player);
    let currentCount = startingnaturalLevel;
    
    // Mark player as in dash countdown
    playersInDashCountdown.add(player.id);
    
    // Clear manual bonus to prevent natural progression from interfering
    playernaturalManualBonus.set(player.id, 0);
    
    console.log(`Starting dash countdown for ${player.name} from ${startingnaturalLevel} to 0 (${isSecondDash ? 'Second' : 'First'} dash)`);
    
    // Create countdown interval
    const countdownInterval = system.runInterval(() => {
        try {
            // Decrease count
            currentCount--;
            
            // Update player's hit count
            playernaturalProgress.set(player.id, currentCount);
            
            // Clear old tags and add new tag
            clearAllnaturalTags(player);
            addnaturalTag(player, currentCount);
            
            // Update action bar if player is still holding the sword
            const equipment = player.getComponent('minecraft:equippable');
            if (equipment) {
                const mainHandItem = equipment.getEquipment('Mainhand');
                if (mainHandItem && mainHandItem.typeId === 'amarite:amarite_longsword=double_dash') {
                    playersWithActionBar.set(player.id, currentCount);
                }
            }
            
            console.log(`Player ${player.name} dash countdown: ${currentCount}`);
            
            // Check if countdown is complete
            if (currentCount <= 0) {
                // Clear the interval
                system.clearRun(countdownInterval);
                
                // Remove from countdown tracking
                playersInDashCountdown.delete(player.id);
                
                // Remove from dash cooldown
                playersDashCooldown.delete(player.id);
                
                // Clean up second dash eligibility
                playersEligibleForSecondDash.delete(player.id);
                
                // Reset natural progression timer AND clear manual bonus
                playernaturalStartTime.set(player.id, system.currentTick);
                playernaturalManualBonus.set(player.id, 0);
                
                console.log(`Player ${player.name} dash countdown completed`);
            }
        } catch (error) {
            console.warn('Error in dash countdown:', error);
            system.clearRun(countdownInterval);
            playersInDashCountdown.delete(player.id);
            playersDashCooldown.delete(player.id);
            playersEligibleForSecondDash.delete(player.id);
        }
    }, 2); // Countdown every 2 ticks
}

// Function to perform dash
function performDash(player, isSecondDash = false) {
    const naturalLevel = isSecondDash ? 
        playersEligibleForSecondDash.get(player.id) || 0 : 
        getPlayernaturalLevel(player);
    
    // Check if player can dash (hit count >= 1)
    if (naturalLevel < 1) {
        return false;
    }
    
    // For second dash, check if player is eligible
    if (isSecondDash && !playersEligibleForSecondDash.has(player.id)) {
        return false;
    }
    
    // Check if player is already in dash cooldown or countdown (but allow second dash during first dash)
    if (!isSecondDash && (playersDashCooldown.has(player.id) || playersInDashCountdown.has(player.id))) {
        return false;
    }
    
    try {
        // Get player's view direction
        const { x, y, z } = player.getViewDirection();
        
        // Apply dash effect using knockback (scale with hit count for more powerful dash)
        const dashPower = Math.min(1 + (naturalLevel * 0.5), 15); // Cap at 15 for balance
        player.applyKnockback(x, z, dashPower, 1.2);
        
        // Play sound effect
        player.playSound("jump.snow", { volume: 0.1, pitch: isSecondDash ? 0.7 : 0.5 });
        
        // Create continuous spherical particle effect that follows the player
        const dashDuration = 20; // Duration in ticks (1 second)
        const particleRadius = 2.5; // Radius of the sphere
        let dashTick = 0;
        
        const dashParticleInterval = system.runInterval(() => {
            try {
                dashTick++;
                
                // Create spherical particle pattern
                const particlesPerTick = 15; // Number of particles per tick
                const playerLocation = player.location;
                
                for (let i = 0; i < particlesPerTick; i++) {
                    // Generate points on a sphere using spherical coordinates
                    const phi = Math.random() * Math.PI * 2; // Azimuthal angle (0 to 2π)
                    const cosTheta = Math.random() * 2 - 1; // cos(θ) where θ is polar angle
                    const sinTheta = Math.sqrt(1 - cosTheta * cosTheta);
                    
                    // Convert to Cartesian coordinates
                    const sphereX = particleRadius * sinTheta * Math.cos(phi);
                    const sphereY = particleRadius * cosTheta;
                    const sphereZ = particleRadius * sinTheta * Math.sin(phi);
                    
                    // Add some randomness to the radius for a more organic look
                    const radiusVariation = 0.8 + Math.random() * 0.4; // 0.8 to 1.2 multiplier
                    
                    const particleLocation = {
                        x: playerLocation.x + (sphereX * radiusVariation),
                        y: playerLocation.y + 1 + (sphereY * radiusVariation), // Offset by 1 to center on player
                        z: playerLocation.z + (sphereZ * radiusVariation)
                    };
                    
                    player.dimension.spawnParticle('minecraft:endrod', particleLocation);
                }
                
                // Add some inner sphere particles for density
                for (let i = 0; i < 8; i++) {
                    const phi = Math.random() * Math.PI * 2;
                    const cosTheta = Math.random() * 2 - 1;
                    const sinTheta = Math.sqrt(1 - cosTheta * cosTheta);
                    const innerRadius = particleRadius * 0.5; // Smaller inner sphere
                    
                    const sphereX = innerRadius * sinTheta * Math.cos(phi);
                    const sphereY = innerRadius * cosTheta;
                    const sphereZ = innerRadius * sinTheta * Math.sin(phi);
                    
                    const particleLocation = {
                        x: playerLocation.x + sphereX,
                        y: playerLocation.y + 1 + sphereY,
                        z: playerLocation.z + sphereZ
                    };
                    
                    player.dimension.spawnParticle('minecraft:endrod', particleLocation);
                }
                
                // Stop the particle effect when dash duration is over
                if (dashTick >= dashDuration) {
                    system.clearRun(dashParticleInterval);
                    console.log(`Dash particle effect ended for ${player.name}`);
                    
                    // If this was the first dash, set up second dash eligibility
                    if (!isSecondDash) {
                        playersInFirstDash.delete(player.id);
                        // Store the hit count for potential second dash
                        playersEligibleForSecondDash.set(player.id, naturalLevel);
                        
                        // Set up a timeout to remove second dash eligibility after a short window
                        system.runTimeout(() => {
                            // Only remove eligibility if player hasn't used second dash
                            if (playersEligibleForSecondDash.has(player.id)) {
                                console.log(`Second dash window expired for ${player.name}`);
                            }
                        }, 40); // 2 second window for second dash
                    }
                }
                
            } catch (error) {
                console.warn('Error spawning dash particles:', error);
                system.clearRun(dashParticleInterval);
            }
        }, 1); // Run every tick for smooth particle effect
        
        if (!isSecondDash) {
            // Mark as in first dash
            playersInFirstDash.add(player.id);
            // Add to dash cooldown
            playersDashCooldown.add(player.id);
        } else {
            // Second dash - start countdown immediately
            performDashCountdown(player, true);
            // Remove second dash eligibility
            playersEligibleForSecondDash.delete(player.id);
        }
        
        console.log(`Player ${player.name} performed ${isSecondDash ? 'second' : 'first'} dash with ${naturalLevel} hit count`);
        return true;
        
    } catch (error) {
        console.warn('Error performing dash:', error);
        if (!isSecondDash) {
            playersDashCooldown.delete(player.id);
            playersInFirstDash.delete(player.id);
        }
        return false;
    }
}

// Function to start action bar display for a player
function startActionBarDisplay(player, naturalLevel) {
    // Clear any existing interval
    if (actionBarIntervals.has(player.id)) {
        system.clearRun(actionBarIntervals.get(player.id));
    }
    
    // Store that this player should have action bar displayed
    playersWithActionBar.set(player.id, naturalLevel);
    
    // Set up interval to continuously display action bar while holding item
    const intervalId = system.runInterval(() => {
        try {
            // Check if player is still holding the Amarite Longsword
            const equipment = player.getComponent('minecraft:equippable');
            if (!equipment) {
                clearActionBar(player);
                return;
            }
            
            const mainHandItem = equipment.getEquipment('Mainhand');
            if (!mainHandItem || mainHandItem.typeId !== 'amarite:amarite_longsword=double_dash') {
                clearActionBar(player);
                return;
            }
            
            // Get current hit count for this player
            const currentnaturalLevel = playersWithActionBar.get(player.id) || 0;
            // Always show action bar when holding the sword
            player.onScreenDisplay.setActionBar(getActionBarText(currentnaturalLevel, player));
        } catch (error) {
            console.warn('Error updating action bar:', error);
            clearActionBar(player);
        }
    }, 2); // Update every 2 ticks (0.1 seconds) to prevent fading
    
    actionBarIntervals.set(player.id, intervalId);
}

// Function to check if player should restore action bar when switching back to item
function checkAndRestoreActionBar(player) {
    const equipment = player.getComponent('minecraft:equippable');
    if (!equipment) return;
    
    const mainHandItem = equipment.getEquipment('Mainhand');
    if (mainHandItem && mainHandItem.typeId === 'amarite:amarite_longsword=double_dash') {
        // Always show action bar when holding the sword, even with 0 hits
        const naturalLevel = getPlayernaturalLevel(player);
        startActionBarDisplay(player, naturalLevel);
    }
}

// Subscribe to item start use events
world.afterEvents.itemStartUse.subscribe((eventData) => {
    const { source, itemStack } = eventData;
    
    // Check if the used item is the Amarite Longsword
    if (itemStack.typeId === 'amarite:amarite_longsword=double_dash') {
        playersUsingItems.add(source.id);
        // Initialize natural progression if not already started
        if (!playernaturalProgress.has(source.id)) {
            playernaturalProgress.set(source.id, 0);
            playernaturalStartTime.set(source.id, system.currentTick);
            addnaturalTag(source, 0);
        }
        // Show action bar with current hit count
        const currentnaturalLevel = getPlayernaturalLevel(source);
        startActionBarDisplay(source, currentnaturalLevel);
        console.log(`Player ${source.name} started using Amarite Longsword`);
    }
});

// Subscribe to item stop use events
world.afterEvents.itemStopUse.subscribe((eventData) => {
    const { source, itemStack } = eventData;
    
    // Check if the stopped item is the Amarite Longsword
    if (itemStack && itemStack.typeId === 'amarite:amarite_longsword=double_dash') {
        playersUsingItems.delete(source.id);
        // Clear action bar display
        clearActionBar(source);
        console.log(`Player ${source.name} stopped using Amarite Longsword`);
    }
});

// Subscribe to item complete use events (backup)
world.afterEvents.itemCompleteUse.subscribe((eventData) => {
    const { source, itemStack } = eventData;
    
    // Check if the completed item is the Amarite Longsword
    if (itemStack.typeId === 'amarite:amarite_longsword=double_dash') {
        playersUsingItems.delete(source.id);
        // Clear action bar display
        clearActionBar(source);
        console.log(`Player ${source.name} completed using Amarite Longsword`);
    }
});

// Subscribe to entity hurt events
world.afterEvents.entityHurt.subscribe((eventData) => {
    const { hurtEntity, damageSource, damage } = eventData;
    
    // Check if the hurt entity is a player
    if (hurtEntity.typeId !== 'minecraft:player') {
        return;
    }
    
    // Check if damage source is from another entity
    if (!damageSource.damagingEntity) {
        return;
    }
    
    // Check if the player is currently using an item (blocking)
    if (!playersUsingItems.has(hurtEntity.id)) {
        return;
    }
    
    // Get the player's equipment
    const equipment = hurtEntity.getComponent('minecraft:equippable');
    if (!equipment) {
        return;
    }
    
    // Check if player is holding the Amarite Longsword in main hand
    const mainHandItem = equipment.getEquipment('Mainhand');
    if (!mainHandItem || mainHandItem.typeId !== 'amarite:amarite_longsword=double_dash') {
        return;
    }
    
    // Don't increment hit count if player is in dash countdown
    if (playersInDashCountdown.has(hurtEntity.id)) {
        return;
    }
    
    // Increase natural progress when blocking attacks
    increaseNaturalProgress(hurtEntity, 1); // Increase by 2 when blocking attacks
    
    const newnaturalLevel = getPlayernaturalLevel(hurtEntity);
    
    console.log(`Player ${hurtEntity.name} blocked damage while using Amarite Longsword (Level ${newnaturalLevel})`);
    
    // Calculate reduced damage (half damage)
    const reducedDamage = damage * 0.5;
    const damageReduction = damage - reducedDamage;
    
    // Store player's current velocity to restore it (reducing knockback)
    const currentVelocity = hurtEntity.getVelocity();
    
    // Heal the player for the damage reduction amount
    // This effectively reduces the damage they took
    system.run(() => {
        try {
            const healthComponent = hurtEntity.getComponent('minecraft:health');
            if (healthComponent) {
                const currentHealth = healthComponent.currentValue;
                const maxHealth = healthComponent.effectiveMax;
                const newHealth = Math.min(currentHealth + damageReduction, maxHealth);
                
                healthComponent.setCurrentValue(newHealth);
                
                // Optional: Add visual/audio feedback
                hurtEntity.runCommand(`playsound Longsword_Absorbs_Damage @a[r=6]`);
                
                // Start action bar display that persists while holding item
                startActionBarDisplay(hurtEntity, newnaturalLevel);
            }
            
            // Reduce knockback by applying counter-velocity
            // Wait a tick to let the knockback apply first, then counter it
            system.runTimeout(() => {
                try {
                    const newVelocity = hurtEntity.getVelocity();
                    
                    // Calculate the knockback that was applied
                    const knockbackX = newVelocity.x - currentVelocity.x;
                    const knockbackY = newVelocity.y - currentVelocity.y;
                    const knockbackZ = newVelocity.z - currentVelocity.z;
                    
                    // Reduce knockback by 70% (keep only 30% of the knockback)
                    const knockbackReduction = 1.0;
                    const counterX = -knockbackX * knockbackReduction;
                    const counterY = -knockbackY * knockbackReduction;
                    const counterZ = -knockbackZ * knockbackReduction;
                    
                    // Apply the counter-velocity to reduce knockback
                    hurtEntity.applyKnockback(0, 0, counterX, counterY + counterZ);
                    
                } catch (error) {
                    console.warn('Error reducing knockback:', error);
                }
            });
            
        } catch (error) {
            console.warn('Error applying damage reduction:', error);
        }
    });
});

// Handle item use event for dash
world.afterEvents.itemUse.subscribe((eventData) => {
    const player = eventData.source;
    const item = eventData.itemStack;
    
    if (item?.typeId === "amarite:amarite_longsword=double_dash" && !player.hasTag("jump_cooldown")) {
        const naturalLevel = getPlayernaturalLevel(player);
        
        // Check if player has at least 1 hit to dash AND is sneaking
        if (naturalLevel >= 1 && player.isSneaking) {
            // Check if player is eligible for second dash
            if (playersEligibleForSecondDash.has(player.id)) {
                const dashSuccess = performDash(player, true); // Second dash
                
                if (dashSuccess) {
                    player.addTag("jump_cooldown");
                    
                    system.runTimeout(() => {
                        player.removeTag("jump_cooldown");
                    }, 10);
                }
            } else if (!playersInFirstDash.has(player.id) && !playersDashCooldown.has(player.id)) {
                // First dash
                const dashSuccess = performDash(player, false);
                
                if (dashSuccess) {
                    player.addTag("jump_cooldown");
                    
                    // Start countdown after first dash completes
                    system.runTimeout(() => {
                        if (!playersEligibleForSecondDash.has(player.id)) {
                            // Only start countdown if second dash wasn't used
                            performDashCountdown(player, false);
                        }
                    }, 20); // Wait for dash duration to complete
                    
                    system.runTimeout(() => {
                        player.removeTag("jump_cooldown");
                    }, 10);
                }
            }
        }
    }
});

// Subscribe to player interact events to detect left-click while right-clicking (backup method)
world.beforeEvents.itemUse.subscribe((eventData) => {
    const { source, itemStack } = eventData;
    
    // Check if the item is the Amarite Longsword
    if (itemStack.typeId === 'amarite:amarite_longsword=double_dash') {
        // Check if player is currently blocking (right-clicking)
        if (playersUsingItems.has(source.id)) {
            // This is a left-click while right-clicking (dash attempt)
            const naturalLevel = getPlayernaturalLevel(source);
            
            // Check for second dash eligibility
            if (playersEligibleForSecondDash.has(source.id) && source.isSneaking && !source.hasTag("jump_cooldown")) {
                const dashSuccess = performDash(source, true);
                if (dashSuccess) {
                    source.addTag("jump_cooldown");
                    
                    system.runTimeout(() => {
                        source.removeTag("jump_cooldown");
                    }, 2);
                    
                    eventData.cancel = true;
                }
            }
            // Check for first dash
            else if (naturalLevel >= 1 && !source.hasTag("jump_cooldown") && !playersInDashCountdown.has(source.id) && !playersInFirstDash.has(source.id) && !playersDashCooldown.has(source.id) && source.isSneaking) {
                const dashSuccess = performDash(source, false);
                if (dashSuccess) {
                    source.addTag("jump_cooldown");
                    
                    // Start countdown after first dash completes
                    system.runTimeout(() => {
                        if (!playersEligibleForSecondDash.has(source.id)) {
                            performDashCountdown(source, false);
                        }
                    }, 20);
                    
                    system.runTimeout(() => {
                        source.removeTag("jump_cooldown");
                    }, 10);
                    
                    eventData.cancel = true;
                }
            }
        }
    }
});

// Natural progression system - runs continuously for all players holding the sword
system.runInterval(() => {
    try {
        for (const player of world.getPlayers()) {
            // Skip players who are in dash countdown
            if (playersInDashCountdown.has(player.id)) {
                continue;
            }
            
            const equipment = player.getComponent('minecraft:equippable');
            if (equipment) {
                const mainHandItem = equipment.getEquipment('Mainhand');
                if (mainHandItem && mainHandItem.typeId === 'amarite:amarite_longsword=double_dash') {
                    // Update natural progression for this player
                    const oldLevel = getPlayernaturalLevel(player);
                    updatePlayernatural(player);
                    const newLevel = getPlayernaturalLevel(player);
                    
                    // Update action bar if level changed
                    if (oldLevel !== newLevel) {
                        playersWithActionBar.set(player.id, newLevel);
                        console.log(`Player ${player.name} natural progression: ${oldLevel} -> ${newLevel}`);
                    }
                    
                    // Ensure action bar is displayed
                    if (!actionBarIntervals.has(player.id)) {
                        startActionBarDisplay(player, newLevel);
                    }
                }
            }
        }
    } catch (error) {
        console.warn('Error in natural progression system:', error);
    }
}, 5); // Update every 5 ticks for smooth progression

// Monitor players switching items to restore action bar when needed
system.runInterval(() => {
    try {
        for (const player of world.getPlayers()) {
            // Check if player is holding the sword but doesn't have active action bar
            const equipment = player.getComponent('minecraft:equippable');
            if (equipment) {
                const mainHandItem = equipment.getEquipment('Mainhand');
                if (mainHandItem && mainHandItem.typeId === 'amarite:amarite_longsword=double_dash') {
                    const hasActiveActionBar = actionBarIntervals.has(player.id);
                    if (!hasActiveActionBar) {
                        // Show action bar when holding the sword
                        const naturalLevel = getPlayernaturalLevel(player);
                        startActionBarDisplay(player, naturalLevel);
                    }
                }
            }
        }
    } catch (error) {
        console.warn('Error in item monitoring:', error);
    }
}, 10); // Check every 10 ticks (0.5 seconds)

// Clean up intervals when players leave
world.afterEvents.playerLeave.subscribe((eventData) => {
    const { playerId } = eventData;
    
    // Clean up tracking data
    playersHeardChargedSound.delete(playerId);
    playersUsingItems.delete(playerId);
    playernaturalProgress.delete(playerId);
    playernaturalStartTime.delete(playerId);
    playernaturalManualBonus.delete(playerId); // Add this line
    playersWithActionBar.delete(playerId);
    playersDashCooldown.delete(playerId);
    playersInDashCountdown.delete(playerId);
    playersEligibleForSecondDash.delete(playerId);
    playersInFirstDash.delete(playerId);
    
    // Clear action bar interval
    if (actionBarIntervals.has(playerId)) {
        system.clearRun(actionBarIntervals.get(playerId));
        actionBarIntervals.delete(playerId);
    }
});

// Optional: Add a function to reset hit counts and tags (useful for testing or resetting)
export function resetPlayernaturalLevel(player) {
    resetPlayernatural(player);
    playersDashCooldown.delete(player.id);
    playersInDashCountdown.delete(player.id);
    playersEligibleForSecondDash.delete(player.id);
    playersInFirstDash.delete(player.id);
    playersHeardChargedSound.delete(player.id);
    console.log(`Reset natural progression for player ${player.name}`);
}
