import { system, world, ItemComponentTypes, ItemStack, EquipmentSlot } from "@minecraft/server";

// Array of all mask types
const maskTypes = [
  "amarite:axolotl_mask",
  "amarite:bunny_mask",
  "amarite:butterfly_mask",
  "amarite:crown_mask",
  "amarite:demon_mask",
  "amarite:fox_mask",
  "amarite:guardian_mask",
  "amarite:moon_mask",
  "amarite:oni_mask",
  "amarite:unicorn_mask",
  "amarite:warden_mask",
  "amarite:winsweep_mask"
];

world.beforeEvents.itemUse.subscribe((event) => {
  const { itemStack, source } = event;
  
  if (itemStack && maskTypes.includes(itemStack.typeId)) {
    event.cancel = true;
    
    system.run(() => {
      const equipmentComp = source.getComponent("equippable");
      const inventory = source.getComponent("inventory").container;
      
      if (equipmentComp && inventory) {
        // Get current head item FIRST
        const currentHeadItem = equipmentComp.getEquipment(EquipmentSlot.Head);
        
        // Find the SPECIFIC mask being used in inventory and replace it with the current head item
        for (let i = 0; i < inventory.size; i++) {
          const slot = inventory.getItem(i);
          if (slot && slot.typeId === itemStack.typeId) {
            inventory.setItem(i, currentHeadItem);
            break;
          }
        }
        
        // THEN run the command to equip the mask
        source.runCommand(`replaceitem entity @s slot.armor.head 1 ${itemStack.typeId}`);
        
        // Play the armor equip sound
        source.runCommand("playsound armor.equip_leather @s");
        
      }
    });
  }
});
