import { world, system } from '@minecraft/server';

const TAG_NAME = "mask_on_player_on_head_has_tag_mask_on_player_on_head_has_tag_concealment";
const DISPLAY_NAME = "";
const MASK_ITEMS = [
    "amarite:axolotl_mask",
    "amarite:bunny_mask",
    "amarite:butterfly_mask",
    "amarite:crown_mask",
    "amarite:demon_mask",
    "amarite:fox_mask",
    "amarite:guardian_mask",
    "amarite:moon_mask",
    "amarite:oni_mask",
    "amarite:unicorn_mask",
    "amarite:warden_mask",
    "amarite:winsweep_mask"
];

// Run every 20 ticks (1 second)
system.runInterval(() => {
    try {
        for (const player of world.getPlayers()) {
            checkMaskAndRemoveTag(player);
            updatePlayerNameTag(player);
        }
    } catch (error) {
        console.warn("Error in nametag script:", error);
    }
});

function checkMaskAndRemoveTag(player) {
    try {
        if (player && player.isValid()) {
            const equipment = player.getComponent('minecraft:equippable');
            if (equipment) {
                const headSlot = equipment.getEquipment('Head');
                const hasMask = headSlot && MASK_ITEMS.includes(headSlot.typeId);
                
                // Remove tag if not wearing any of the specified masks
                if (!hasMask && player.hasTag(TAG_NAME)) {
                    player.removeTag(TAG_NAME);
                }
            }
        }
    } catch (error) {
        console.warn("Error checking mask and removing tag:", error);
    }
}

function updatePlayerNameTag(player) {
    try {
        if (player && player.isValid()) {
            if (player.hasTag(TAG_NAME)) {
                player.nameTag = DISPLAY_NAME;
            } else {
                // Reset to default name
                player.nameTag = player.name;
            }
        }
    } catch (error) {
        console.warn("Error updating player nametag:", error);
    }
}
