import { system, world, ItemComponentTypes, ItemStack } from "@minecraft/server";

const masks = [
  "amarite:axolotl_mask",
  "amarite:bunny_mask",
  "amarite:butterfly_mask",
  "amarite:crown_mask",
  "amarite:demon_mask",
  "amarite:fox_mask",
  "amarite:guardian_mask",
  "amarite:moon_mask",
  "amarite:oni_mask",
  "amarite:unicorn_mask",
  "amarite:warden_mask",
  "amarite:winsweep_mask"
];

const interactionBlocks = [
  // Storage blocks
  "minecraft:chest",
  "minecraft:trapped_chest",
  "minecraft:ender_chest",
  "minecraft:barrel",
  "minecraft:hopper",
  "minecraft:dropper",
  "minecraft:dispenser",
  "minecraft:shulker_box",
  "minecraft:white_shulker_box",
  "minecraft:orange_shulker_box",
  "minecraft:magenta_shulker_box",
  "minecraft:light_blue_shulker_box",
  "minecraft:yellow_shulker_box",
  "minecraft:lime_shulker_box",
  "minecraft:pink_shulker_box",
  "minecraft:gray_shulker_box",
  "minecraft:light_gray_shulker_box",
  "minecraft:cyan_shulker_box",
  "minecraft:purple_shulker_box",
  "minecraft:blue_shulker_box",
  "minecraft:brown_shulker_box",
  "minecraft:green_shulker_box",
  "minecraft:red_shulker_box",
  "minecraft:black_shulker_box",
  
  // Crafting blocks
  "minecraft:crafting_table",
  "minecraft:furnace",
  "minecraft:blast_furnace",
  "minecraft:smoker",
  "minecraft:anvil",
  "minecraft:chipped_anvil",
  "minecraft:damaged_anvil",
  "minecraft:enchanting_table",
  "minecraft:brewing_stand",
  "minecraft:beacon",
  "minecraft:stonecutter",
  "minecraft:loom",
  "minecraft:cartography_table",
  "minecraft:fletching_table",
  "minecraft:smithing_table",
  "minecraft:grindstone",
  "minecraft:composter",
  "minecraft:cauldron",
  "minecraft:water_cauldron",
  "minecraft:lava_cauldron",
  "minecraft:powder_snow_cauldron",
  
  // Interactive utility blocks
  "minecraft:lectern",
  "minecraft:jukebox",
  "minecraft:note_block",
  "minecraft:daylight_detector",
  "minecraft:daylight_detector_inverted",
  "minecraft:observer",
  "minecraft:comparator",
  "minecraft:repeater",
  "minecraft:command_block",
  "minecraft:chain_command_block",
  "minecraft:repeating_command_block",
  "minecraft:structure_block",
  "minecraft:jigsaw",
  
  // Redstone components
  "minecraft:lever",
  "minecraft:stone_button",
  "minecraft:oak_button",
  "minecraft:spruce_button",
  "minecraft:birch_button",
  "minecraft:jungle_button",
  "minecraft:acacia_button",
  "minecraft:dark_oak_button",
  "minecraft:mangrove_button",
  "minecraft:cherry_button",
  "minecraft:bamboo_button",
  "minecraft:crimson_button",
  "minecraft:warped_button",
  "minecraft:polished_blackstone_button",
  "minecraft:stone_pressure_plate",
  "minecraft:oak_pressure_plate",
  "minecraft:spruce_pressure_plate",
  "minecraft:birch_pressure_plate",
  "minecraft:jungle_pressure_plate",
  "minecraft:acacia_pressure_plate",
  "minecraft:dark_oak_pressure_plate",
  "minecraft:mangrove_pressure_plate",
  "minecraft:cherry_pressure_plate",
  "minecraft:bamboo_pressure_plate",
  "minecraft:crimson_pressure_plate",
  "minecraft:warped_pressure_plate",
  "minecraft:light_weighted_pressure_plate",
  "minecraft:heavy_weighted_pressure_plate",
  "minecraft:polished_blackstone_pressure_plate",
  "minecraft:tripwire_hook",
  
  // Doors and gates
  "minecraft:oak_door",
  "minecraft:spruce_door",
  "minecraft:birch_door",
  "minecraft:jungle_door",
  "minecraft:acacia_door",
  "minecraft:dark_oak_door",
  "minecraft:mangrove_door",
  "minecraft:cherry_door",
  "minecraft:bamboo_door",
  "minecraft:crimson_door",
  "minecraft:warped_door",
  "minecraft:iron_door",
  "minecraft:copper_door",
  "minecraft:exposed_copper_door",
  "minecraft:weathered_copper_door",
  "minecraft:oxidized_copper_door",
  "minecraft:waxed_copper_door",
  "minecraft:waxed_exposed_copper_door",
  "minecraft:waxed_weathered_copper_door",
  "minecraft:waxed_oxidized_copper_door",
  
  // Trapdoors
  "minecraft:oak_trapdoor",
  "minecraft:spruce_trapdoor",
  "minecraft:birch_trapdoor",
  "minecraft:jungle_trapdoor",
  "minecraft:acacia_trapdoor",
  "minecraft:dark_oak_trapdoor",
  "minecraft:mangrove_trapdoor",
  "minecraft:cherry_trapdoor",
  "minecraft:bamboo_trapdoor",
  "minecraft:crimson_trapdoor",
  "minecraft:warped_trapdoor",
  "minecraft:iron_trapdoor",
  "minecraft:copper_trapdoor",
  "minecraft:exposed_copper_trapdoor",
  "minecraft:weathered_copper_trapdoor",
  "minecraft:oxidized_copper_trapdoor",
  "minecraft:waxed_copper_trapdoor",
  "minecraft:waxed_exposed_copper_trapdoor",
  "minecraft:waxed_weathered_copper_trapdoor",
  "minecraft:waxed_oxidized_copper_trapdoor",
  
  // Fence gates
  "minecraft:oak_fence_gate",
  "minecraft:spruce_fence_gate",
  "minecraft:birch_fence_gate",
  "minecraft:jungle_fence_gate",
  "minecraft:acacia_fence_gate",
  "minecraft:dark_oak_fence_gate",
  "minecraft:mangrove_fence_gate",
  "minecraft:cherry_fence_gate",
  "minecraft:bamboo_fence_gate",
  "minecraft:crimson_fence_gate",
  "minecraft:warped_fence_gate",
  
  // Beds
  "minecraft:white_bed",
  "minecraft:orange_bed",
  "minecraft:magenta_bed",
  "minecraft:light_blue_bed",
  "minecraft:yellow_bed",
  "minecraft:lime_bed",
  "minecraft:pink_bed",
  "minecraft:gray_bed",
  "minecraft:light_gray_bed",
  "minecraft:cyan_bed",
  "minecraft:purple_bed",
  "minecraft:blue_bed",
  "minecraft:brown_bed",
  "minecraft:green_bed",
  "minecraft:red_bed",
  "minecraft:black_bed",
  
  // Signs
  "minecraft:oak_sign",
  "minecraft:spruce_sign",
  "minecraft:birch_sign",
  "minecraft:jungle_sign",
  "minecraft:acacia_sign",
  "minecraft:dark_oak_sign",
  "minecraft:mangrove_sign",
  "minecraft:cherry_sign",
  "minecraft:bamboo_sign",
  "minecraft:crimson_sign",
  "minecraft:warped_sign",
  "minecraft:oak_hanging_sign",
  "minecraft:spruce_hanging_sign",
  "minecraft:birch_hanging_sign",
  "minecraft:jungle_hanging_sign",
  "minecraft:acacia_hanging_sign",
  "minecraft:dark_oak_hanging_sign",
  "minecraft:mangrove_hanging_sign",
  "minecraft:cherry_hanging_sign",
  "minecraft:bamboo_hanging_sign",
  "minecraft:crimson_hanging_sign",
  "minecraft:warped_hanging_sign",
  
  // Villager workstations
  "minecraft:bell",
  
  // Miscellaneous interactive blocks
  "minecraft:flower_pot",
  "minecraft:decorated_pot",
  "minecraft:suspicious_sand",
  "minecraft:suspicious_gravel",
  "minecraft:trial_spawner",
  "minecraft:vault",
  "minecraft:crafter",
  "minecraft:chiseled_bookshelf"
];

world.beforeEvents.itemUseOn.subscribe((event) => {
  const { itemStack, source, block } = event;
  
  if (itemStack && masks.includes(itemStack.typeId)) {
    // Allow interaction with specific blocks
    if (!interactionBlocks.includes(block.typeId)) {
      event.cancel = true;
    }
  }
});
