import { world } from "@minecraft/server";

// Event for initializing custom component
world.beforeEvents.worldInitialize.subscribe(initEvent => {
    initEvent.blockComponentRegistry.registerCustomComponent('blushrooms:stair_placement', {
        onPlace: e => {
            const { block } = e;
            const adjacentBlocks = {
                north: block.north(),
                south: block.south(),
                east: block.east(),
                west: block.west()
            };

            const verticalHalf = block.permutation.getState('minecraft:vertical_half');
            const shape = block.permutation.getState('blushrooms:shape');
            const cardinal = block.permutation.getState('minecraft:cardinal_direction');

            const applyShape = (currentBlock, tags, direction, currentShape, newShape) => {
                if (
                    currentBlock?.hasTag(tags[0]) &&
                    currentBlock?.hasTag(tags[1]) &&
                    shape === currentShape &&
                    cardinal === direction
                ) {
                    block.setPermutation(
                        block.permutation.withState('blushrooms:shape', newShape)
                    );
                } else if (
                    (!currentBlock?.hasTag(tags[0]) || !currentBlock?.hasTag(tags[1])) &&
                    shape === newShape &&
                    cardinal === direction
                ) {
                    block.setPermutation(
                        block.permutation.withState('blushrooms:shape', 'straight')
                    );
                }
            };

            if (verticalHalf === 'bottom' || verticalHalf === 'top') {
                const positionType = verticalHalf === 'bottom' ? 'custom_stairs_up' : 'custom_stairs_down';
                block.setPermutation(block.permutation.withState('blushrooms:placed_bit', true));

                // Adjustments for inner shapes
                applyShape(adjacentBlocks.south, [positionType, "custom_stairs_east"], "north", "straight", "inner_right");
                applyShape(adjacentBlocks.south, [positionType, "custom_stairs_west"], "north", "straight", "inner_left");
                applyShape(adjacentBlocks.north, [positionType, "custom_stairs_east"], "south", "straight", "inner_left");
                applyShape(adjacentBlocks.north, [positionType, "custom_stairs_west"], "south", "straight", "inner_right");
                applyShape(adjacentBlocks.east, [positionType, "custom_stairs_south"], "west", "straight", "inner_left");
                applyShape(adjacentBlocks.east, [positionType, "custom_stairs_north"], "west", "straight", "inner_right");
                applyShape(adjacentBlocks.west, [positionType, "custom_stairs_north"], "east", "straight", "inner_left");
                applyShape(adjacentBlocks.west, [positionType, "custom_stairs_south"], "east", "straight", "inner_right");

                // Adjustments for outer shapes
                applyShape(adjacentBlocks.north, [positionType, "custom_stairs_east"], "north", "straight", "outer_left");
                applyShape(adjacentBlocks.north, [positionType, "custom_stairs_west"], "north", "straight", "outer_right");
                applyShape(adjacentBlocks.south, [positionType, "custom_stairs_west"], "south", "straight", "outer_left");
                applyShape(adjacentBlocks.south, [positionType, "custom_stairs_east"], "south", "straight", "outer_right");
                applyShape(adjacentBlocks.west, [positionType, "custom_stairs_north"], "west", "straight", "outer_left");
                applyShape(adjacentBlocks.west, [positionType, "custom_stairs_south"], "west", "straight", "outer_right");
                applyShape(adjacentBlocks.east, [positionType, "custom_stairs_south"], "east", "straight", "outer_left");
                applyShape(adjacentBlocks.east, [positionType, "custom_stairs_north"], "east", "straight", "outer_right");


                // -- Cardinal specific adjustments --
                if (cardinal === "east") {
                    // For the south block...
                    const southBlock = adjacentBlocks.south;
                    if (
                        southBlock &&
                        southBlock.permutation.getState("blushrooms:shape") === "straight" &&
                        southBlock.permutation.getState("minecraft:cardinal_direction") === "north"
                    ) {
                        const southEast = southBlock.east();
                        const southWest = southBlock.west();
                        const hasEastWestNeighbor =
                            southEast &&
                            southWest &&
                            (southEast.permutation.getState("minecraft:cardinal_direction") === "north" &&
                                southWest.permutation.getState("minecraft:cardinal_direction") === "north");
                        const hasSpecialShape = ["outer_left", "outer_right", "inner_left", "inner_right"].includes(
                            southBlock.permutation.getState("blushrooms:shape")
                        );
                        if (!hasEastWestNeighbor && !hasSpecialShape) {
                            southBlock.setPermutation(
                                southBlock.permutation
                                    .withState("blushrooms:shape", "outer_left")
                                    .withState("minecraft:cardinal_direction", "north")
                            );
                        }
                    }
                    // For the north block...
                    const northBlock = adjacentBlocks.north;
                    if (
                        northBlock &&
                        northBlock.permutation.getState("blushrooms:shape") === "straight" &&
                        northBlock.permutation.getState("minecraft:cardinal_direction") === "south"
                    ) {
                        const northEast = northBlock.east();
                        const northWest = northBlock.west();
                        const hasEastWestNeighbor =
                            northEast &&
                            northWest &&
                            (northEast.permutation.getState("minecraft:cardinal_direction") === "south" &&
                                northWest.permutation.getState("minecraft:cardinal_direction") === "south");
                        const hasSpecialShape = ["outer_left", "outer_right", "inner_left", "inner_right"].includes(
                            northBlock.permutation.getState("blushrooms:shape")
                        );
                        if (!hasEastWestNeighbor && !hasSpecialShape) {
                            northBlock.setPermutation(
                                northBlock.permutation
                                    .withState("blushrooms:shape", "outer_right")
                                    .withState("minecraft:cardinal_direction", "south")
                            );
                        }
                    }
                }

                if (cardinal === "west") {
                    // For the south block...
                    const southBlock = adjacentBlocks.south;
                    if (
                        southBlock &&
                        southBlock.permutation.getState("blushrooms:shape") === "straight" &&
                        southBlock.permutation.getState("minecraft:cardinal_direction") === "north"
                    ) {
                        const southWest = southBlock.west();
                        const southEast = southBlock.east();
                        const hasWestNeighbor =
                            southWest &&
                            southEast &&
                            (southWest.permutation.getState("minecraft:cardinal_direction") === "north" &&
                                southEast.permutation.getState("minecraft:cardinal_direction") === "north");
                        const hasSpecialShape = ["outer_left", "outer_right", "inner_left", "inner_right"].includes(
                            southBlock.permutation.getState("blushrooms:shape")
                        );
                        if (!hasWestNeighbor && !hasSpecialShape) {
                            southBlock.setPermutation(
                                southBlock.permutation
                                    .withState("blushrooms:shape", "outer_right")
                                    .withState("minecraft:cardinal_direction", "north")
                            );
                        }
                    }
                    // For the north block...
                    const northBlock = adjacentBlocks.north;
                    if (
                        northBlock &&
                        northBlock.permutation.getState("blushrooms:shape") === "straight" &&
                        northBlock.permutation.getState("minecraft:cardinal_direction") === "south"
                    ) {
                        const northWest = northBlock.west();
                        const northEast = northBlock.east();
                        const hasWestNeighbor =
                            northWest &&
                            northEast &&
                            (northWest.permutation.getState("minecraft:cardinal_direction") === "south" &&
                                northEast.permutation.getState("minecraft:cardinal_direction") === "south");
                        const hasSpecialShape = ["outer_left", "outer_right", "inner_left", "inner_right"].includes(
                            northBlock.permutation.getState("blushrooms:shape")
                        );
                        if (!hasWestNeighbor && !hasSpecialShape) {
                            northBlock.setPermutation(
                                northBlock.permutation
                                    .withState("blushrooms:shape", "outer_left")
                                    .withState("minecraft:cardinal_direction", "south")
                            );
                        }
                    }
                }

                if (cardinal === "north") {
                    // For the east block...
                    const eastBlock = adjacentBlocks.east;
                    if (
                        eastBlock &&
                        eastBlock.permutation.getState("blushrooms:shape") === "straight" &&
                        eastBlock.permutation.getState("minecraft:cardinal_direction") === "west"
                    ) {
                        const eastNorth = eastBlock.north();
                        const eastSouth = eastBlock.south();
                        const hasNSNeighbor =
                            eastNorth &&
                            eastSouth &&
                            (eastNorth.permutation.getState("minecraft:cardinal_direction") === "west" &&
                                eastSouth.permutation.getState("minecraft:cardinal_direction") === "west");
                        const hasSpecialShape = ["outer_left", "outer_right", "inner_left", "inner_right"].includes(
                            eastBlock.permutation.getState("blushrooms:shape")
                        );
                        if (!hasNSNeighbor && !hasSpecialShape) {
                            eastBlock.setPermutation(
                                eastBlock.permutation
                                    .withState("blushrooms:shape", "outer_left")
                                    .withState("minecraft:cardinal_direction", "west")
                            );
                        }
                    }
                    // For the west block...
                    const westBlock = adjacentBlocks.west;
                    if (
                        westBlock &&
                        westBlock.permutation.getState("blushrooms:shape") === "straight" &&
                        westBlock.permutation.getState("minecraft:cardinal_direction") === "east"
                    ) {
                        const westNorth = westBlock.north();
                        const westSouth = westBlock.south();
                        const hasNSNeighbor =
                            westNorth &&
                            westSouth &&
                            (westNorth.permutation.getState("minecraft:cardinal_direction") === "east" &&
                                westSouth.permutation.getState("minecraft:cardinal_direction") === "east");
                        const hasSpecialShape = ["outer_left", "outer_right", "inner_left", "inner_right"].includes(
                            westBlock.permutation.getState("blushrooms:shape")
                        );
                        if (!hasNSNeighbor && !hasSpecialShape) {
                            westBlock.setPermutation(
                                westBlock.permutation
                                    .withState("blushrooms:shape", "outer_right")
                                    .withState("minecraft:cardinal_direction", "east")
                            );
                        }
                    }
                }

                if (cardinal === "south") {
                    // For the west block...
                    const westBlock = adjacentBlocks.west;
                    if (
                        westBlock &&
                        westBlock.permutation.getState("blushrooms:shape") === "straight" &&
                        westBlock.permutation.getState("minecraft:cardinal_direction") === "east"
                    ) {
                        const westNorth = westBlock.north();
                        const westSouth = westBlock.south();
                        const hasNSNeighbor =
                            westNorth &&
                            westSouth &&
                            (westNorth.permutation.getState("minecraft:cardinal_direction") === "east" &&
                                westSouth.permutation.getState("minecraft:cardinal_direction") === "east");
                        const hasSpecialShape = ["outer_left", "outer_right", "inner_left", "inner_right"].includes(
                            westBlock.permutation.getState("blushrooms:shape")
                        );
                        if (!hasNSNeighbor && !hasSpecialShape) {
                            westBlock.setPermutation(
                                westBlock.permutation
                                    .withState("blushrooms:shape", "outer_left")
                                    .withState("minecraft:cardinal_direction", "east")
                            );
                        }
                    }
                    // For the east block...
                    const eastBlock = adjacentBlocks.east;
                    if (
                        eastBlock &&
                        eastBlock.permutation.getState("blushrooms:shape") === "straight" &&
                        eastBlock.permutation.getState("minecraft:cardinal_direction") === "west"
                    ) {
                        const eastNorth = eastBlock.north();
                        const eastSouth = eastBlock.south();
                        const hasNSNeighbor =
                            eastNorth &&
                            eastSouth &&
                            (eastNorth.permutation.getState("minecraft:cardinal_direction") === "west" &&
                                eastSouth.permutation.getState("minecraft:cardinal_direction") === "west");
                        const hasSpecialShape = ["outer_left", "outer_right", "inner_left", "inner_right"].includes(
                            eastBlock.permutation.getState("blushrooms:shape")
                        );
                        if (!hasNSNeighbor && !hasSpecialShape) {
                            eastBlock.setPermutation(
                                eastBlock.permutation
                                    .withState("blushrooms:shape", "outer_right")
                                    .withState("minecraft:cardinal_direction", "west")
                            );
                        }
                    }
                }

                // -- Inner corner (cluster) checks --
                if (cardinal === "north") {
                    // Check west inner
                    const westBlock = adjacentBlocks.west;
                    if (
                        westBlock &&
                        westBlock.permutation.getState("blushrooms:shape") === "straight" &&
                        westBlock.permutation.getState("minecraft:cardinal_direction") === "west"
                    ) {
                        const neighbor = westBlock.north();
                        if (!(neighbor && neighbor.permutation.getState("blushrooms:shape") === "straight")) {
                            westBlock.setPermutation(
                                westBlock.permutation
                                    .withState("blushrooms:shape", "inner_right")
                                    .withState("minecraft:cardinal_direction", "west")
                            );
                        }
                    }
                    // Check east inner
                    const eastBlock = adjacentBlocks.east;
                    if (
                        eastBlock &&
                        eastBlock.permutation.getState("blushrooms:shape") === "straight" &&
                        eastBlock.permutation.getState("minecraft:cardinal_direction") === "east"
                    ) {
                        const neighbor = eastBlock.north();
                        if (!(neighbor && neighbor.permutation.getState("blushrooms:shape") === "straight")) {
                            eastBlock.setPermutation(
                                eastBlock.permutation
                                    .withState("blushrooms:shape", "inner_left")
                                    .withState("minecraft:cardinal_direction", "east")
                            );
                        }
                    }
                }
                if (cardinal === "south") {
                    // Check east inner
                    const eastBlock = adjacentBlocks.east;
                    if (
                        eastBlock &&
                        eastBlock.permutation.getState("blushrooms:shape") === "straight" &&
                        eastBlock.permutation.getState("minecraft:cardinal_direction") === "east"
                    ) {
                        const neighbor = eastBlock.south();
                        if (!(neighbor && neighbor.permutation.getState("blushrooms:shape") === "straight")) {
                            eastBlock.setPermutation(
                                eastBlock.permutation
                                    .withState("blushrooms:shape", "inner_right")
                                    .withState("minecraft:cardinal_direction", "east")
                            );
                        }
                    }
                    // Check west inner
                    const westBlock = adjacentBlocks.west;
                    if (
                        westBlock &&
                        westBlock.permutation.getState("blushrooms:shape") === "straight" &&
                        westBlock.permutation.getState("minecraft:cardinal_direction") === "west"
                    ) {
                        const neighbor = westBlock.south();
                        if (!(neighbor && neighbor.permutation.getState("blushrooms:shape") === "straight")) {
                            westBlock.setPermutation(
                                westBlock.permutation
                                    .withState("blushrooms:shape", "inner_left")
                                    .withState("minecraft:cardinal_direction", "west")
                            );
                        }
                    }
                }
                if (cardinal === "east") {
                    // Check south inner
                    const southBlock = adjacentBlocks.south;
                    if (
                        southBlock &&
                        southBlock.permutation.getState("blushrooms:shape") === "straight" &&
                        southBlock.permutation.getState("minecraft:cardinal_direction") === "south"
                    ) {
                        const neighbor = southBlock.east();
                        if (!(neighbor && neighbor.permutation.getState("blushrooms:shape") === "straight")) {
                            southBlock.setPermutation(
                                southBlock.permutation
                                    .withState("blushrooms:shape", "inner_left")
                                    .withState("minecraft:cardinal_direction", "south")
                            );
                        }
                    }
                    // Check north inner
                    const northBlock = adjacentBlocks.north;
                    if (
                        northBlock &&
                        northBlock.permutation.getState("blushrooms:shape") === "straight" &&
                        northBlock.permutation.getState("minecraft:cardinal_direction") === "north"
                    ) {
                        const neighbor = northBlock.east();
                        if (!(neighbor && neighbor.permutation.getState("blushrooms:shape") === "straight")) {
                            northBlock.setPermutation(
                                northBlock.permutation
                                    .withState("blushrooms:shape", "inner_right")
                                    .withState("minecraft:cardinal_direction", "north")
                            );
                        }
                    }
                }
                if (cardinal === "west") {
                    // Check south inner
                    const southBlock = adjacentBlocks.south;
                    if (
                        southBlock &&
                        southBlock.permutation.getState("blushrooms:shape") === "straight" &&
                        southBlock.permutation.getState("minecraft:cardinal_direction") === "south"
                    ) {
                        const neighbor = southBlock.west();
                        if (!(neighbor && neighbor.permutation.getState("blushrooms:shape") === "straight")) {
                            southBlock.setPermutation(
                                southBlock.permutation
                                    .withState("blushrooms:shape", "inner_right")
                                    .withState("minecraft:cardinal_direction", "south")
                            );
                        }
                    }
                    // Check north inner
                    const northBlock = adjacentBlocks.north;
                    if (
                        northBlock &&
                        northBlock.permutation.getState("blushrooms:shape") === "straight" &&
                        northBlock.permutation.getState("minecraft:cardinal_direction") === "north"
                    ) {
                        const neighbor = northBlock.west();
                        if (!(neighbor && neighbor.permutation.getState("blushrooms:shape") === "straight")) {
                            northBlock.setPermutation(
                                northBlock.permutation
                                    .withState("blushrooms:shape", "inner_left")
                                    .withState("minecraft:cardinal_direction", "north")
                            );
                        }
                    }
                }
            }
        }
    });
});

world.afterEvents.playerBreakBlock.subscribe(event => {
    const { block } = event;
    if (!block) return;

    const neighbors = {
        north: block.north?.(),
        south: block.south?.(),
        east: block.east?.(),
        west: block.west?.()
    };

    // Revert any specialized shape settings on adjacent blocks when a block is broken.
    Object.values(neighbors).forEach(neighbor => {
        if (!neighbor) return;
        const nShape = neighbor.permutation?.getState("blushrooms:shape");
        if (["inner_left", "inner_right", "outer_left", "outer_right"].includes(nShape)) {
            neighbor.setPermutation(neighbor.permutation.withState("blushrooms:shape", "straight"));
        }
    });
});